//package eu.eudat.model.builder.filetransformer;
//
//import eu.eudat.authorization.AuthorizationFlags;
//import eu.eudat.commons.enums.IsActive;
//import eu.eudat.convention.ConventionService;
//import eu.eudat.data.*;
//import eu.eudat.file.transformer.enums.DmpAccessType;
//import eu.eudat.file.transformer.enums.DmpStatus;
//import eu.eudat.file.transformer.enums.DmpVersionStatus;
//import eu.eudat.file.transformer.models.description.DescriptionFileTransformerModel;
//import eu.eudat.file.transformer.models.dmp.DmpFileTransformerModel;
//import eu.eudat.file.transformer.models.dmp.DmpReferenceFileTransformerModel;
//import eu.eudat.file.transformer.models.dmp.DmpUserFileTransformerModel;
//import eu.eudat.file.transformer.models.dmpblueprint.DmpBlueprintFileTransformerModel;
//import eu.eudat.file.transformer.models.dmpblueprint.definition.SectionFileTransformerModel;
//import eu.eudat.file.transformer.models.entitydoi.EntityDoiFileTransformerModel;
//import eu.eudat.file.transformer.models.reference.ReferenceFileTransformerModel;
//import eu.eudat.file.transformer.models.user.UserFileTransformerModel;
//import eu.eudat.model.*;
//import eu.eudat.model.builder.DmpReferenceBuilder;
//import eu.eudat.query.*;
//import gr.cite.tools.data.builder.BuilderFactory;
//import gr.cite.tools.data.query.Ordering;
//import gr.cite.tools.data.query.QueryFactory;
//import gr.cite.tools.exception.MyApplicationException;
//import gr.cite.tools.fieldset.BaseFieldSet;
//import gr.cite.tools.logging.LoggerService;
//import org.slf4j.LoggerFactory;
//import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.beans.factory.config.ConfigurableBeanFactory;
//import org.springframework.context.annotation.Scope;
//import org.springframework.stereotype.Component;
//
//import java.time.Instant;
//import java.util.*;
//import java.util.stream.Collectors;
//
//@Component
//@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
//public class DmpFileTransformerBuilder extends BaseFileTransformerBuilder<DmpFileTransformerModel, DmpEntity> {
//
//    private final QueryFactory queryFactory;
//    private final BuilderFactory builderFactory;
//    private EnumSet<AuthorizationFlags> authorize = EnumSet.of(AuthorizationFlags.None);
//
//    private Boolean ignoreDescriptions = false;
//
//    @Autowired
//    public DmpFileTransformerBuilder(ConventionService conventionService,
//                                     QueryFactory queryFactory,
//                                     BuilderFactory builderFactory) {
//        super(conventionService, new LoggerService(LoggerFactory.getLogger(DmpFileTransformerBuilder.class)));
//        this.queryFactory = queryFactory;
//        this.builderFactory = builderFactory;
//    }
//
//    public DmpFileTransformerBuilder authorize(EnumSet<AuthorizationFlags> values) {
//        this.authorize = values;
//        return this;
//    }
//
//    public DmpFileTransformerBuilder ignoreDescriptions(Boolean ignoreDescriptions) {
//        this.ignoreDescriptions = ignoreDescriptions;
//        return this;
//    }
//
//    @Override
//    protected List<FileTransformerBuilderItemResponse<DmpFileTransformerModel, DmpEntity>> buildInternal(List<DmpEntity> data) throws MyApplicationException {
//        this.logger.debug("building for {}", Optional.ofNullable(data).map(List::size).orElse(0));
//        if (data == null || data.isEmpty()) return new ArrayList<>();
//
//        List<FileTransformerBuilderItemResponse<DmpFileTransformerModel, DmpEntity>> models = new ArrayList<>();
//
//        Map<UUID, List<DmpReferenceFileTransformerModel>> dmpReferencesMap = this.collectReferences(data);
//
//        Map<UUID, List<DmpUserFileTransformerModel>> dmpUsersMap = this.collectDmpUsers(data);
//
//        Map<UUID, List<DescriptionFileTransformerModel>> descriptionsMap = null;
//        if (!ignoreDescriptions) {
//            descriptionsMap = this.collectDmpDescriptions(data);
//        }
//
//        Map<UUID, List<EntityDoiFileTransformerModel>> doiMap = this.collectEntityDois(data);
//        Map<UUID, UserFileTransformerModel> creatorMap = this.collectCreators(data);
//        Map<UUID, DmpBlueprintFileTransformerModel> blueprintMap = this.collectBlueprints(data);
//
//        for (DmpEntity d : data) {
//            DmpFileTransformerModel m = new DmpFileTransformerModel();
//            m.setId(d.getId());
//            m.setLabel(d.getLabel());
//            m.setVersion(d.getVersion());
//            m.setDescription(d.getDescription());
//            m.setFinalizedAt(d.getFinalizedAt());
//            m.setCreatedAt(d.getCreatedAt());
//            if (d.getPublicAfter() != null && d.getPublicAfter().isAfter(Instant.now())) {
//                m.setPublishedAt(d.getPublicAfter());
//            }
//            switch (d.getAccessType()){
//                case Public -> m.setAccessType(DmpAccessType.Public);
//                case Restricted -> m.setAccessType(DmpAccessType.Restricted);
//                case null -> m.setAccessType(null);
//                default -> throw new MyApplicationException("unrecognized type " + d.getAccessType().getValue());
//            }
//
//            m.setProperties(d.getProperties());
//            m.setUpdatedAt(d.getUpdatedAt());
//            m.setLanguage(d.getLanguage());
//            m.setStatus(DmpStatus.of(d.getStatus().getValue()));
//            m.setVersionStatus(DmpVersionStatus.of(d.getVersionStatus().getValue()));
//            if (dmpReferencesMap != null && !dmpReferencesMap.isEmpty() && dmpReferencesMap.containsKey(d.getId())) m.setDmpReferences(dmpReferencesMap.get(d.getId()));
//            if (dmpUsersMap != null && !dmpUsersMap.isEmpty() && dmpUsersMap.containsKey(d.getId())) m.setDmpUsers(dmpUsersMap.get(d.getId()));
//            if (descriptionsMap != null && !descriptionsMap.isEmpty() && descriptionsMap.containsKey(d.getId())) m.setDescriptions(descriptionsMap.get(d.getId()));
//            if (doiMap != null && !doiMap.isEmpty() && doiMap.containsKey(d.getId())) m.setEntityDois(doiMap.get(d.getId()));
//            if (creatorMap != null && !creatorMap.isEmpty() && creatorMap.containsKey(d.getId())) m.setCreator(creatorMap.get(d.getId()));
//            if (blueprintMap != null && !blueprintMap.isEmpty() && blueprintMap.containsKey(d.getBlueprintId())) m.setBlueprint(blueprintMap.get(d.getBlueprintId()));
//
//            SectionFileTransformerModel templateSection = m.getBlueprint().getDefinitionFileTransformerModel().getSections().stream().filter(SectionFileTransformerModel::getHasTemplates).findFirst().orElse(null);
//            if (templateSection != null && m.getDescriptions() != null && !m.getDescriptions().isEmpty()) {
//                m.getDescriptions().forEach(description -> {
//                    description.setSectionId(templateSection.getId());
//                    description.setCreatedBy(m.getCreator());
//                });
//            }
//
//            models.add(new FileTransformerBuilderItemResponse<>(m, d));
//        }
//        this.logger.debug("build {} items", Optional.of(models).map(List::size).orElse(0));
//
//        return models;
//    }
//
//    private Map<UUID, List<DmpReferenceFileTransformerModel>> collectReferences(List<DmpEntity> data) throws MyApplicationException {
//        if (data.isEmpty()) return null;
//        this.logger.debug("checking related - {}", DmpReference.class.getSimpleName());
//
//        List<DmpReferenceEntity> dmpReferences = this.queryFactory.query(DmpReferenceQuery.class).isActives(IsActive.Active).authorize(this.authorize).dmpIds(data.stream().map(DmpEntity::getId).distinct().collect(Collectors.toList())).collectAs(new BaseFieldSet().ensure(DmpReference._dmp).ensure(DmpReference._reference));
//
//        Map<UUID, List<DmpReferenceFileTransformerModel>> itemMap = new HashMap<>();
//        ReferenceQuery query = this.queryFactory.query(ReferenceQuery.class).authorize(this.authorize).isActive(IsActive.Active).ids(dmpReferences.stream().map(DmpReferenceEntity::getReferenceId).distinct().collect(Collectors.toList()));
//        Map<UUID, ReferenceFileTransformerModel> referenceDepositModelMap = this.builderFactory.builder(ReferenceFileTransformerBuilder.class).authorize(this.authorize).asForeignKey(query, ReferenceEntity::getId);
//        if (referenceDepositModelMap == null) return null;
//        for (DmpReferenceEntity dmpReference : dmpReferences) {
//            DmpReferenceFileTransformerModel dmpReferenceFileTransformerModel = new DmpReferenceFileTransformerModel();
//            dmpReferenceFileTransformerModel.setId(dmpReference.getId());
//            dmpReferenceFileTransformerModel.setData(dmpReference.getData());
//            dmpReferenceFileTransformerModel.setReference(referenceDepositModelMap.getOrDefault(dmpReference.getReferenceId(), null));
//            DmpReferenceFileTransformerModel model  = dmpReferenceFileTransformerModel;
//            if (model == null) continue;
//            UUID key = dmpReference.getDmpId();
//            if (!itemMap.containsKey(key)) itemMap.put(key, new ArrayList<>());
//            itemMap.get(key).add(model);
//        }
//        return itemMap;
//    }
//
//
//    private Map<UUID, List<DmpUserFileTransformerModel>> collectDmpUsers(List<DmpEntity> data) throws MyApplicationException {
//        this.logger.debug("checking related - {}", DmpUser.class.getSimpleName());
//
//        Map<UUID, List<DmpUserFileTransformerModel>> itemMap;
//        DmpUserQuery query = this.queryFactory.query(DmpUserQuery.class).isActives(IsActive.Active).authorize(this.authorize).dmpIds(data.stream().map(DmpEntity::getId).distinct().collect(Collectors.toList()));
//        itemMap = this.builderFactory.builder(DmpUserFileTransformerBuilder.class).authorize(this.authorize).asMasterKey(query, DmpUserEntity::getDmpId);
//
//        return itemMap;
//    }
//
//    private Map<UUID, List<DescriptionFileTransformerModel>> collectDmpDescriptions(List<DmpEntity> data) throws MyApplicationException {
//        if (data.isEmpty()) return null;
//        this.logger.debug("checking related - {}", Description.class.getSimpleName());
//
//        Map<UUID, List<DescriptionFileTransformerModel>> itemMap;
//        DescriptionQuery query = this.queryFactory.query(DescriptionQuery.class).isActive(IsActive.Active).authorize(this.authorize).dmpIds(data.stream().map(DmpEntity::getId).distinct().collect(Collectors.toList()));
//        itemMap = this.builderFactory.builder(DescriptionFileTransformerBuilder.class).authorize(this.authorize).asMasterKey(query, DescriptionEntity::getDmpId);
//
//        return itemMap;
//    }
//
//    private Map<UUID, UserFileTransformerModel> collectCreators(List<DmpEntity> data) {
//        DmpUserQuery query = this.queryFactory.query(DmpUserQuery.class).isActives(IsActive.Active).authorize(this.authorize).userIds(data.stream().map(DmpEntity::getCreatorId).toList());
//        return this.builderFactory.builder(DmpUserFileTransformerBuilder.class).authorize(this.authorize).asMasterKey(query, DmpUserEntity::getDmpId)
//                .entrySet().stream().map(uuidListEntry -> Map.entry(uuidListEntry.getKey(), uuidListEntry.getValue().get(0).getUser()))
//                .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
//    }
//
//    private Map<UUID, DmpBlueprintFileTransformerModel> collectBlueprints(List<DmpEntity> data) {
//        DmpBlueprintQuery query = this.queryFactory.query(DmpBlueprintQuery.class).isActive(IsActive.Active).authorize(this.authorize).ids(data.stream().map(DmpEntity::getBlueprintId).toList());
//        return this.builderFactory.builder(DmpBlueprintFileTransformerBuilder.class).authorize(this.authorize).asMasterKey(query, DmpBlueprintEntity::getId)
//                .entrySet().stream().map(uuidListEntry -> Map.entry(uuidListEntry.getKey(), uuidListEntry.getValue().get(0)))
//                .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
//    }
//
//    private Map<UUID, List<EntityDoiFileTransformerModel>> collectEntityDois(List<DmpEntity> data) {
//        if (data.isEmpty()) return null;
//        Map<UUID, List<EntityDoiFileTransformerModel>> result;
//        EntityDoiQuery query = this.queryFactory.query(EntityDoiQuery.class).isActive(IsActive.Active).authorize(authorize).entityIds(data.stream().map(DmpEntity::getId).distinct().toList());
//        result = builderFactory.builder(EntityDoiFileTransformerBuilder.class).asMasterKey(query, EntityDoiEntity::getEntityId);
//        return result;
//    }
//
//}
